#!/usr/bin/env node
"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const chalk_1 = __importDefault(require("chalk"));
const commander_1 = require("commander");
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const update_notifier_1 = __importDefault(require("update-notifier"));
const buildCommand_1 = __importDefault(require("../commands/buildCommand"));
const cacheCommands_1 = require("../commands/cacheCommands");
const convertSketch_1 = __importDefault(require("../commands/convertSketch"));
const createSketch_1 = __importDefault(require("../commands/createSketch"));
const screenshotCommand_1 = __importDefault(require("../commands/screenshotCommand"));
const serveCommand_1 = __importDefault(require("../commands/serveCommand"));
const proxyCache_1 = require("../server/proxyCache");
const program = new commander_1.Command();
const PROJECT_HOME = path_1.default.join(__dirname, '../../');
const P5_ANALYSIS_BIN = path_1.default.join(PROJECT_HOME, 'node_modules/.bin');
const pkg = JSON.parse(fs_1.default.readFileSync(path_1.default.join(PROJECT_HOME, 'package.json'), 'utf-8'));
program.version(pkg.version);
(0, update_notifier_1.default)({ pkg }).notify({
    isGlobal: true,
    message: `Update available ${chalk_1.default.dim('{currentVersion}')} ${chalk_1.default.reset('→')} ${chalk_1.default.green('{latestVersion}')}
  Run ${chalk_1.default.cyan('{updateCommand}')} to update
  Changes: ${chalk_1.default.blue('https://bit.ly/p5-server-changelog')}`
});
program
    .command('create')
    .description('Create a new p5.js sketch')
    .alias('c')
    .alias('generate')
    .alias('g')
    .argument('[FILE]', 'the filename of the sketch', 'sketch.js')
    .option('--type <TYPE>', 'place the new files in a folder')
    .option('--force', 'force overwrite of existing files')
    .option('--options [OPTIONS]', 'comma-separated list of template options')
    .option('-t, --title [TITLE]', 'sketch title')
    .action(createSketch_1.default);
program
    .command('build')
    .description('Create a static site that presents the sketches')
    .alias('b')
    .argument('[SOURCE]', 'the source directory', '.')
    .option('-o, --output <OUTPUT>', 'the output directory', 'build')
    .option('--open', 'Open the index file in a browser')
    .option('--options [OPTIONS]', 'comma-separated list of template options')
    .option('-t, --theme <FILE>', 'template file', 'split')
    .option('-v, --verbose', 'verbose output')
    .option('--dry-run', 'dry run')
    .action(buildCommand_1.default);
program
    .command('convert')
    .description('Convert an HTML sketch to JavaScript-only or vice versa')
    .argument('FILE')
    .option('--to <type>', 'output type: script, html, folder')
    .action(convertSketch_1.default);
program
    .command('screenshot')
    .argument('SKETCH_FILE')
    .option('-o, --output <OUTPUT>', 'the output file')
    .option('--browser <NAME>', 'safari | chrome | firefox | edge')
    .option('--canvas-size <DIMENSIONS>', 'canvas dimensions e.g. 100 or 100,200')
    .option('--frame-count <NUMBER>', 'the number of frames saved', '1')
    .option('--pixel-density <NUMBER>', 'e.g. 2, 0.5, or 1/2')
    .option('--skip-frames <COUNT>', 'omit the first COUNT frames')
    .action(screenshotCommand_1.default);
program
    .command('serve')
    .argument('[DIRECTORY...]', 'the directory to serve', '.')
    .description('Start the web server')
    .alias('server')
    .alias('r')
    .alias('run')
    .alias('r')
    .option('-o, --open', 'Open the page in a browser')
    .option('-p, --port [PORT]', 'HTTP port to listen on', '3000')
    .option('-t, --theme [FILE]', 'template file')
    .option('--browser <NAME>', 'safari | chrome | firefox | edge (implies --open)')
    .option('--split', 'Use the split (directory + sketch) template')
    .option('--no-proxy-cache', 'disable the CDN proxy cache')
    .option('--console [FORMAT]', 'Relay console messages and errors to sketch in the server console')
    .action(serveCommand_1.default);
/*
 * Proxy Cache subcommands
 */
const cacheCommand = program.command('proxy-cache');
cacheCommand
    .command('clear')
    .description('Clear the cache')
    .action(async () => {
    await proxyCache_1.cacache.rm.all(proxyCache_1.cachePath);
    console.log(`Cache cleared`);
});
cacheCommand
    .command('info')
    .alias('stats')
    .option('--json', 'Output JSON')
    .description('Print summmary information about the cache')
    .action(cacheCommands_1.printCacheInfo);
cacheCommand
    .command('list')
    .alias('ls')
    .option('--json', 'Output JSON')
    .option('-v, --verbose', 'Print (some) metadata')
    .description('List the cache entries')
    .action(cacheCommands_1.lsCache);
cacheCommand
    .command('path')
    .description('Print the path to the cache')
    .action(() => {
    console.log(proxyCache_1.cachePath);
});
cacheCommand
    .command('warm')
    .alias('fill')
    .description('Fill the cache from known library import paths')
    .option('--force', 'Force refresh of cached entries')
    .option('-v, --verbose', 'verbose output')
    .action(cacheCommands_1.fillCache);
/*
 * Subcommands
 */
program.command('analyze', 'Display information about a sketch', {
    executableFile: `${P5_ANALYSIS_BIN}/p5-analyze`
});
for (const command of ['library', 'libraries']) {
    program.command(command, 'Print information about p5.js libraries known to p5-server', {
        executableFile: `${P5_ANALYSIS_BIN}/p5-libraries`
    });
}
program.command('tree', 'Print the tree structure of a directory and its sketches', {
    executableFile: `${P5_ANALYSIS_BIN}/p5-tree`
});
program.parse(process.argv);
