"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = __importDefault(require("fs"));
const promises_1 = require("fs/promises");
const minimatch_1 = __importDefault(require("minimatch"));
const open_1 = __importDefault(require("open"));
const p5_analysis_1 = require("p5-analysis");
const path_1 = __importDefault(require("path"));
const directoryListing_1 = require("../server/directoryListing");
const templates_1 = require("../server/templates");
const helpers_1 = require("../helpers");
const chalk_1 = __importDefault(require("chalk"));
const directoryExclusions = [...directoryListing_1.defaultDirectoryExclusions, 'build'];
async function build(source, options) {
    const output = options.output;
    const hrstart = process.hrtime.bigint();
    if (options.theme === 'directory') {
        options.theme = 'grid';
        process.stderr.write(chalk_1.default.yellow('The "directory" theme has been renamed to grid. A future release will remove the "directory" value.\n'));
    }
    if ((0, helpers_1.pathIsInDirectory)(output, source) &&
        !directoryExclusions.some(pattern => (0, minimatch_1.default)(output, pattern))) {
        (0, helpers_1.die)('The output directory cannot be inside the source directory');
    }
    if ((0, helpers_1.pathIsInDirectory)(source, output)) {
        (0, helpers_1.die)('The source directory cannot be inside the output directory');
    }
    if (!options.dryRun && fs_1.default.existsSync(output)) {
        fs_1.default.readdirSync(output)
            .filter(file => !file.startsWith('.'))
            .map(file => path_1.default.join(output, file))
            .forEach(file => fs_1.default.statSync(file).isDirectory()
            ? (0, promises_1.rm)(file, { recursive: true })
            : fs_1.default.unlinkSync(file));
    }
    const actions = createActions(source, output);
    const count = await runActions(actions, options);
    const rootIndex = path_1.default.join(output, 'index.html');
    const elapsed = Number(process.hrtime.bigint() - hrstart) / 1e9;
    console.log(`p5 build wrote ${count} files to directory ${output} in ${elapsed.toFixed(2)}ms`);
    if (options.open) {
        (0, open_1.default)(rootIndex);
    }
    else if (options.verbose) {
        console.log(`Open file://${path_1.default.resolve(rootIndex)} to view`);
    }
}
exports.default = build;
function Action(kind, source, outputFile) {
    return { kind, source, outputFile };
}
function createActions(file, output) {
    return visit(file, output);
    async function* visit(source, output) {
        if (fs_1.default.statSync(source).isDirectory()) {
            yield* visitDir(source, output);
        }
        else {
            yield Action('copyFile', source, output);
            if ((0, helpers_1.pathIsMarkdown)(source)) {
                yield Action('convertMarkdown', source, output + '.html');
            }
        }
    }
    async function* visitDir(dir, output) {
        const { sketches, allFiles } = await p5_analysis_1.Sketch.analyzeDirectory(dir, {
            exclusions: directoryExclusions
        });
        yield Action('mkdir', dir, output);
        const scriptOnlySketches = sketches.filter(sk => sk.structureType === 'script');
        // TODO: check for collisions when choosing the output file path
        for (const sketch of scriptOnlySketches) {
            const outputFile = path_1.default
                .join(output, sketch.scriptFile)
                .replace(/\.js$/i, '.html');
            yield { kind: 'createSketchHtml', sketch, outputFile };
        }
        const subdirectorySketches = sketches.filter(sk => sk.dir !== dir);
        for (const sketch of subdirectorySketches) {
            yield Action('copyDir', sketch.dir, path_1.default.join(output, path_1.default.relative(dir, sketch.dir)));
        }
        // Do this after the directories are copied
        for (const sketch of sketches) {
            const outputFile = path_1.default
                .join(output, path_1.default.relative(dir, sketch.scriptFilePath))
                .replace(/\.js$/i, '.js.html');
            yield { kind: 'createSourceView', source: sketch.scriptFilePath, outputFile };
        }
        for (const file of allFiles) {
            yield* visit(path_1.default.join(dir, file), path_1.default.join(output, file));
        }
        if (!allFiles.find(file => /^index\.html?$/i.test(file))) {
            // Generate the index from the target, rather than the source, so that it
            // will refer to generated HTML files instead of the bare JavaScript
            // sketches.
            const outputFile = path_1.default.join(output, 'index.html');
            // construct the listing from the output directory, because it contains
            // the generated sketch HTML files
            yield {
                kind: 'createIndex',
                dir,
                outputFile,
                path: path_1.default.basename(dir)
            };
        }
    }
}
async function runActions(actions, options) {
    let filesCreated = 0;
    for await (const action of actions) {
        if (options.verbose || options.dryRun) {
            const args = actionMessageArgs(action);
            if (args)
                console.log(...args);
        }
        if (!options.dryRun) {
            const { filesCreated: n } = await runAction(action);
            filesCreated += n;
        }
    }
    return filesCreated;
    function actionMessageArgs(action) {
        const { outputFile } = action;
        switch (action.kind) {
            case 'copyDir':
                return ['Copy directory', action.source, '->', outputFile];
            case 'copyFile':
                return ['Copy file', action.source, '->', outputFile];
            case 'convertMarkdown':
                return ['Convert', action.source, '->', outputFile];
            case 'createIndex':
                return ['Generate directory listing', outputFile];
            case 'createSketchHtml':
                return ['Generate sketch HTML', action.sketch.scriptFile, '->', outputFile];
            case 'createSourceView':
                return ['Create source view', action.source, '->', outputFile];
            case 'mkdir':
                if (!fs_1.default.existsSync(outputFile)) {
                    return ['Create directory', outputFile];
                }
                break;
        }
    }
    async function runAction(action) {
        const { outputFile } = action;
        let filesCreated = 0;
        switch (action.kind) {
            case 'copyDir':
                fs_1.default.cpSync(action.source, outputFile, { recursive: true });
                filesCreated += fs_1.default.readdirSync(action.source).length; // FIXME
                break;
            case 'copyFile':
                fs_1.default.copyFileSync(action.source, outputFile);
                filesCreated += 1;
                break;
            case 'convertMarkdown': {
                const html = (0, templates_1.markdownToHtmlPage)(fs_1.default.readFileSync(action.source, 'utf-8'));
                fs_1.default.writeFileSync(outputFile, html);
                filesCreated += 1;
                break;
            }
            case 'createIndex': {
                const { dir, path } = action;
                fs_1.default.rmSync(outputFile, { force: true });
                const templateOptions = (0, helpers_1.stringToOptions)(options.options);
                const html = await (0, directoryListing_1.createDirectoryListing)(dir, path, {
                    staticMode: true,
                    templateName: options.theme,
                    templateOptions
                });
                await (0, promises_1.writeFile)(outputFile, html);
                filesCreated += 1;
                break;
            }
            case 'createSketchHtml': {
                const { sketch } = action;
                const html = await sketch.getHtmlContent();
                await (0, promises_1.writeFile)(outputFile, html);
                filesCreated += 1;
                break;
            }
            case 'createSourceView': {
                const source = fs_1.default.readFileSync(action.source, 'utf-8');
                const title = path_1.default.basename(action.source);
                const html = (0, templates_1.sourceViewTemplate)({ source, title });
                await (0, promises_1.writeFile)(outputFile, html);
                filesCreated += 1;
                break;
            }
            case 'mkdir':
                fs_1.default.mkdirSync(outputFile, { recursive: true });
                break;
        }
        return { filesCreated };
    }
}
