"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const chalk_1 = __importDefault(require("chalk"));
const util_1 = __importDefault(require("util"));
const helpers_1 = require("../helpers");
const Server_1 = require("../server/Server");
async function serve(files, options) {
    // Compatability warnings for old options
    if (options.split) {
        if (options.theme && options.theme !== 'split') {
            (0, helpers_1.die)('Use either --split or --theme but not both');
        }
        process.stderr.write(chalk_1.default.yellow('Warning: --split is now the default. This option will be removed in a future release.\n'));
    }
    if (options.theme === 'directory') {
        options.theme = 'grid';
        process.stderr.write(chalk_1.default.yellow('The "directory" theme has been renamed to grid. A future release will remove the "directory" value.\n'));
    }
    const file = files[0] || '.';
    const displayName = file === '.' ? process.cwd() : file;
    const serverOptions = {
        proxyCache: options.proxyCache,
        port: Number(options.port),
        root: file,
        relayConsoleMessages: Boolean(options.console) && options.console !== 'passive',
        theme: options.theme || undefined
    };
    if (files.length > 1)
        serverOptions.mountPoints = files;
    const server = await Server_1.Server.start(serverOptions);
    if (options.console)
        subscribeToBrowserEvents(server, options.console === 'json');
    console.log(`Serving ${displayName} at ${server.url}`);
    if ((options.open || options.browser) && server.url)
        (0, helpers_1.openInBrowser)(server.url, options.browser?.toLowerCase());
}
exports.default = serve;
function subscribeToBrowserEvents(server, asJson) {
    const consoleColors = {
        debug: chalk_1.default.blueBright,
        error: chalk_1.default.red,
        info: chalk_1.default.green,
        log: chalk_1.default.gray,
        warn: chalk_1.default.yellow,
        clear: null
    };
    server.onScriptEvent('connection', (data) => {
        console.error(chalk_1.default.italic.dim(`browser connection: ${data.type}`), makeLocationString(data));
    });
    server.onScriptEvent('console', (data) => {
        if (asJson) {
            console.log('browser console:', data);
        }
        else {
            const { method, args, argStrings } = data;
            const argsOrStrings = args.map((str, i) => argStrings[i] ?? str);
            const message = typeof args[0] === 'string'
                ? util_1.default.format(...argsOrStrings)
                : argsOrStrings.join(' ');
            const color = consoleColors[method] || chalk_1.default.black;
            console.log(color(`browser console.${method}${args.length ? ': ' : ''}${message}`), makeLocationString(data));
        }
    });
    server.onScriptEvent('document', (data) => {
        const { type, visibilityState } = data;
        console.error(chalk_1.default.italic.dim(`browser document.${type}: ${visibilityState}`), makeLocationString(data));
    });
    server.onScriptEvent('error', (data) => {
        if (asJson) {
            console.log('browser error:', data);
        }
        else {
            const { type, message, stack } = data;
            console.error(chalk_1.default.italic.bold.red(`browser ${type}: ${message}`), makeLocationString(data));
            if (stack)
                console.error(chalk_1.default.red(stack.replace(/^/gm, '  ')));
        }
    });
    server.onScriptEvent('window', (data) => {
        if (asJson) {
            console.log('browser window event:', data);
        }
        else {
            console.log(chalk_1.default.italic.dim.blue(`browser window.${data.type}`), makeLocationString(data));
        }
    });
    function makeLocationString(data) {
        const { clientId, file, url } = data;
        let loc = file || url;
        if (data.type === 'console' || data.type === 'error') {
            const { col, line } = data;
            loc += col && line ? `:${line}:${col}` : line ? `:${line}` : '';
        }
        return chalk_1.default.dim(`(${loc} – ${clientId})`);
    }
}
