"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.addScriptToHtmlHead = exports.stringToOptions = exports.pathIsMarkdown = exports.pathIsInDirectory = exports.pathComponentsForBreadcrumbs = exports.openInBrowser = exports.escapeHTML = exports.die = void 0;
const chalk_1 = __importDefault(require("chalk"));
const node_html_parser_1 = require("node-html-parser");
const open_1 = __importDefault(require("open"));
const path_1 = __importDefault(require("path"));
/** Print the message to standard output; then exit with status code 1.
 */
function die(message) {
    console.error(chalk_1.default.red('Error:', message));
    process.exit(1);
}
exports.die = die;
/** Escape HTML special characters in a string.
 */
function escapeHTML(str) {
    return str
        .replace(/&/g, '&amp;')
        .replace(/</g, '&lt;')
        .replace(/>/g, '&gt;')
        .replace(/"/g, '&quot;')
        .replace(/'/g, '&#39;');
}
exports.escapeHTML = escapeHTML;
/**
 *	Open a URL in the browser.
 *
 * This is a wrapper for `open()` from the 'open' package. It has the same API,
 *	 except that it also accepts 'safari' as an app name.
 */
function openInBrowser(url, browser) {
    const appName = browser === 'safari'
        ? 'safari'
        : browser in open_1.default.apps
            ? browser
            : undefined;
    if (browser && !browser) {
        die(`Unknown browser: ${browser}`);
    }
    const openApps = { safari: 'safari', ...open_1.default.apps };
    const openOptions = appName ? { app: { name: openApps[appName] } } : {};
    return (0, open_1.default)(url, openOptions);
}
exports.openInBrowser = openInBrowser;
/**
 * 'a/b/c' => [{name: 'Home', path: '/'}, {name: 'a', path: '/a'}, {name: 'b', path: '/a/b'}, {name: 'c', path: '/a/b/c'}]
 */
function pathComponentsForBreadcrumbs(relDirPath) {
    // normalize the path: remove the final '/' if it exists
    return (relDirPath
        .replace(/\/$/, '')
        .split('/')
        // skip the first element. We're going to use an init arg to reduce()
        // instead.
        .slice(1)
        .reduce((crumbs, name) => [
        ...crumbs,
        {
            name,
            path: (crumbs[crumbs.length - 1].path + '/').replace('//', '/') + name
        }
    ], [{ name: 'Home', path: '/' }]));
}
exports.pathComponentsForBreadcrumbs = pathComponentsForBreadcrumbs;
/** Tests whether filepath is inside the directory `dir`. */
function pathIsInDirectory(filepath, dir) {
    return !(path_1.default.relative(filepath, dir) + path_1.default.sep).startsWith('..' + path_1.default.sep);
}
exports.pathIsInDirectory = pathIsInDirectory;
/** Returns true iff pathname ends in a markdown file suffix. */
function pathIsMarkdown(filepath) {
    return /\.(md|mkd|mkdn|mdwn|mdown|markdown)$/i.test(filepath);
}
exports.pathIsMarkdown = pathIsMarkdown;
/**
 * Examples:
 * 'a,b,c' => {a: true, b: true, c: true}
 * 'a,no-b,c' => {a: true, b: false, c: true}
 */
function stringToOptions(str) {
    return str
        ? Object.fromEntries(str.split(',').map(s => [s.replace(/^no-/, ''), !s.startsWith('no-')]))
        : {};
}
exports.stringToOptions = stringToOptions;
let warnedAboutMissingHtmlBody = false;
/** Insert a <script> element in an HTML document's head.
 *
 * If the source argument is a string, it becomes the value of the element's
 * `src` attribute.
 *
 * If it is an object with a key `script` it becomes the text content of the
 * element.
 *
 * Otherwise it is an Object; the script tag defines its keys as global
 * variables, that are initialized to the corresponding values.
 */
function addScriptToHtmlHead(html, source) {
    const htmlRoot = (0, node_html_parser_1.parse)(html);
    const scriptNode = new node_html_parser_1.HTMLElement('script', {}, typeof source === 'string' ? `src=${JSON.stringify(source)}` : '', null);
    if (source instanceof Object) {
        scriptNode.textContent =
            'script' in source
                ? source.script
                : Object.entries(source)
                    .map(([k, v]) => `const ${k} = ${JSON.stringify(v)};`)
                    .join('\n');
    }
    // The following works during development. Previously it has failed in distr:
    // htmlRoot.querySelector(tagName) always returns null.
    if (!htmlRoot.querySelector('head')) {
        const body = htmlRoot.querySelector('body');
        if (body)
            body.appendChild(new node_html_parser_1.HTMLElement('head', {}, '', null));
        else if (!warnedAboutMissingHtmlBody) {
            console.warn('HTML document did not have a body');
            warnedAboutMissingHtmlBody = true;
        }
    }
    const head = htmlRoot.querySelector('head');
    if (!head) {
        return html.replace(/(<\/head>)/, '$1' + scriptNode.outerHTML);
    }
    head.appendChild(scriptNode);
    return htmlRoot.outerHTML;
}
exports.addScriptToHtmlHead = addScriptToHtmlHead;
