"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Server = void 0;
const express_1 = __importDefault(require("express"));
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const pug_1 = __importDefault(require("pug"));
const stream_1 = require("stream");
const ts_extras_1 = require("../ts-extras");
const browserScriptEventRelay_1 = require("./browserScriptEventRelay");
const proxyCache_1 = require("./proxyCache");
const constants_1 = require("./constants");
const directoryListing_1 = require("./directoryListing");
const httpServerUtils_1 = require("./httpServerUtils");
const liveReload_1 = require("./liveReload");
const routes_1 = require("./routes");
const templates_1 = require("./templates");
const chalk = require("chalk");
const defaultServerOptions = {
    liveServer: true,
    logConsoleEvents: false,
    port: 3000,
    proxyCache: true,
    relayConsoleMessages: false,
    scanPorts: true,
    screenshot: null,
    theme: 'split',
};
async function startServer(config, sketchRelay) {
    const mountPoints = config.mountPoints;
    const app = (0, express_1.default)();
    // app.use((req, res, next) => {
    //   console.log('', req.method, req.originalUrl);
    //   next();
    // });
    // add routes
    app.use(constants_1.staticAssetPrefix, express_1.default.static(path_1.default.join(__dirname, 'static')));
    for (const { filePath, urlPath } of mountPoints) {
        let root = filePath;
        let sketchFile;
        if (!fs_1.default.statSync(root).isDirectory()) {
            sketchFile = root;
            root = path_1.default.dirname(root);
        }
        const routerConfig = { ...config, root, sketchFile };
        app.use(urlPath, (0, routes_1.createRouter)(routerConfig));
        app.use(urlPath, express_1.default.static(root));
    }
    if (mountPoints.every(mp => mp.urlPath !== '/')) {
        const mountListTmpl = pug_1.default.compileFile(path_1.default.join(templates_1.templateDir, 'mountPoints.pug'));
        app.get('/', (_req, res) => {
            res.send(mountListTmpl({ mountPoints, staticAssetPrefix: constants_1.staticAssetPrefix, path: path_1.default }));
        });
    }
    app.use(proxyCache_1.proxyPrefix, proxyCache_1.cdnProxyRouter);
    app.use((req, _res, next) => {
        console.warn(chalk.red(`Not found (404): ${req.originalUrl}`));
        next();
    });
    // For effect only. This provide errors and diagnostics before waiting for a
    // browser request.
    if (fs_1.default.statSync(mountPoints[0].filePath).isDirectory()) {
        (0, directoryListing_1.createDirectoryListing)(mountPoints[0].filePath, mountPoints[0].urlPath, {
            templateName: config.theme,
        });
    }
    // Scan for an avialable port
    let server = null;
    const port = config.port;
    for (let p = port; p < port + 10; p++) {
        try {
            server = await (0, httpServerUtils_1.promiseListen)(app, p);
            break; // success!
        }
        catch (err) {
            (0, ts_extras_1.assertError)(err);
            if (err.code !== 'EADDRINUSE' || !config.scanPorts) {
                throw err;
            }
            console.log(`Port ${p} is in use, retrying...`);
        }
    }
    // If the port scan didn't find an available port within the range. Allow
    // server.listen to choose a port.
    if (!server)
        server = await (0, httpServerUtils_1.promiseListen)(app);
    const address = server.address();
    if (!address || typeof address === 'string') {
        throw new Error('Failed to start the server');
    }
    (0, browserScriptEventRelay_1.attachBrowserScriptRelay)(server, sketchRelay);
    try {
        const liveReloadServer = await (0, liveReload_1.createLiveReloadServer)({
            port: Math.min(port + 35729 - config.port, 30000),
            scanPorts: true,
            watchDirs: [templates_1.templateDir, ...mountPoints.map(mount => mount.filePath)],
        });
        app.locals.liveReloadServer = liveReloadServer;
        const url = `http://localhost:${address.port}`;
        return { server, liveReloadServer, url };
    }
    catch (e) {
        server.close();
        throw e;
    }
}
/** Server is a web server with live reload, sketch-aware directory listings,
 * and library inference for JavaScript-only sketches.
 */
class Server {
    constructor(options = {}) {
        var _a;
        this.server = null;
        this.liveReloadServer = null;
        this.browserScriptEmitter = new stream_1.EventEmitter();
        this.emitScriptEvent = this.browserScriptEmitter.emit.bind(this.browserScriptEmitter);
        this.onScriptEvent = this.browserScriptEmitter.on.bind(this.browserScriptEmitter);
        const mountPoints = options.mountPoints && options.mountPoints.length > 0
            ? Server._normalizeMountPoints(options.mountPoints)
            : [{ filePath: options.root || '.', urlPath: '/' }];
        this.mountPoints = mountPoints;
        // null out the root. It is only used in initialization, and is now captured
        // in mountPoints instead.
        this.config = { ...defaultServerOptions, root: null, ...options, mountPoints };
        (_a = this.config).theme || (_a.theme = defaultServerOptions.theme);
    }
    /** Create and start the server. Returns the instance. */
    static async start(options = {}) {
        return new Server(options).start();
    }
    async start() {
        const { server, liveReloadServer, url } = await startServer(this.config, this);
        this.server = server;
        this.liveReloadServer = liveReloadServer;
        this.url = url;
        return this;
    }
    /**
     * Close the server and the liveServer.
     *
     * Note: Can return before the liveServer is stopped.
     */
    async close() {
        if (this.server) {
            await (0, httpServerUtils_1.promiseClose)(this.server);
            this.server = null;
        }
        this.url = undefined;
        this.liveReloadServer?.close();
        this.liveReloadServer = null;
    }
    filePathToUrl(filePath) {
        const baseUrl = this.url || `http://localhost:${this.config.port}`;
        for (const mountPoint of this.mountPoints) {
            const filePrefix = mountPoint.filePath + path_1.default.sep;
            const pathPrefix = mountPoint.urlPath.replace(/(?<!\/)$/, '/');
            if (filePath.startsWith(filePrefix)) {
                return baseUrl + filePath.replace(filePrefix, pathPrefix);
            }
        }
        return null;
    }
    urlPathToFilePath(urlPath) {
        for (const mountPoint of this.mountPoints) {
            const filePrefix = mountPoint.filePath + path_1.default.sep;
            const pathPrefix = mountPoint.urlPath.replace(/(?<!\/)$/, '/');
            if (urlPath.startsWith(pathPrefix)) {
                return urlPath.replace(pathPrefix, filePrefix);
            }
        }
        return null;
    }
    serverUrlToFileUrl(url) {
        const baseUrl = this.url || `http://localhost:${this.config.port}`;
        if (url.startsWith(baseUrl + '/')) {
            const filepath = this.urlPathToFilePath(url.slice(baseUrl.length));
            if (filepath)
                return 'file://' + path_1.default.resolve(filepath);
        }
        return null;
    }
    /** Normalize file paths; remove trailing slashes from file and url paths;
     * generate unique names and prefixes for mount points that don't specify
     * them. */
    static _normalizeMountPoints(mountPoints) {
        const finalPathSep = new RegExp(`${path_1.default.sep}$`);
        const mounts = mountPoints
            // normalize to records
            .map(mount => (typeof mount === 'string' ? { filePath: mount } : mount))
            // default url paths from file paths
            .map(mount => ({
            urlPath: '/' + (mount.name || path_1.default.basename(mount.filePath)),
            ...mount,
        }))
            // encode URL paths
            .map(mount => ({ ...mount, urlPath: mount.urlPath.replace(/ /g, ' ') }))
            // normalize Windows paths
            .map(mount => ({ ...mount, filePath: mount.filePath.replace(/\//g, path_1.default.sep) }))
            // remove trailing slashes from file and url paths
            .map(mount => ({
            ...mount,
            filePath: mount.filePath.replace(finalPathSep, ''),
            urlPath: mount.urlPath.replace(/\/$/, ''),
        }));
        // modify url paths to ensure that they are unique
        const seen = new Set();
        for (const mount of mounts) {
            if (seen.has(mount.urlPath)) {
                mount.urlPath = findUniqueName(mount.urlPath, seen);
            }
            seen.add(mount.urlPath);
        }
        return mounts;
        function findUniqueName(base, exclude) {
            for (const name of generateNames(base)) {
                if (!exclude.has(name)) {
                    return name;
                }
            }
            throw new Error('This should not happen');
        }
        function* generateNames(base) {
            yield base;
            let ix = 2;
            const m = base.match(/^(.*?)-(\d*)$/);
            if (m) {
                base = m[1];
                ix = parseInt(m[2], 10) + 1;
            }
            while (true) {
                yield `${base}-${ix++}`;
            }
        }
    }
}
exports.Server = Server;
