"use strict";
/** Receive and parse messages from the client-side console relay, and emit them
 * as events. */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.injectScriptEventRelayScript = exports.replaceUrlsInStack = exports.attachBrowserScriptRelay = void 0;
const url_1 = require("url");
const ws_1 = __importDefault(require("ws"));
const helpers_1 = require("../helpers");
const jsonCycleStringifier_1 = require("../jsonCycleStringifier");
const ts_extras_1 = require("../ts-extras");
const constants_1 = require("./constants");
const { parse: parseCyclicJson } = (0, jsonCycleStringifier_1.jsonCycleStringifier)();
function attachBrowserScriptRelay(server, relay) {
    const handlers = new Map();
    const wsServer = new ws_1.default.Server({ noServer: true });
    wsServer.on('connection', socket => {
        socket.on('message', message => {
            const [route, data] = parseCyclicJson(message.toString());
            const handler = handlers.get(route);
            if (handler) {
                handler({
                    ...data,
                    file: urlToFilePath(data.url) || undefined,
                    stack: replaceUrlsInStack(relay, data.stack),
                    timestamp: new Date(data.timestamp),
                });
            }
        });
    });
    server.on('upgrade', (request, socket, head) => {
        wsServer.handleUpgrade(request, socket, head, socket => {
            wsServer.emit('connection', socket, request);
        });
    });
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    function defineHandler(route, handler) {
        handlers.set(route, handler);
    }
    defineHandler('connection', (message) => {
        const event = message;
        relay.emitScriptEvent('connection', event);
    });
    defineHandler('console', (message) => {
        const event = { type: 'console', argStrings: [], ...message };
        const args = event.args.map(decodeUnserializableValue);
        // const argStrings = event.argStrings || [];
        const data = { ...event, type: 'console', args };
        relay.emitScriptEvent('console', data);
    });
    defineHandler('document', (message) => {
        const event = message;
        relay.emitScriptEvent('document', event);
    });
    defineHandler('error', (message) => {
        const event = message;
        relay.emitScriptEvent('error', event);
    });
    defineHandler('window', (message) => {
        const event = message;
        relay.emitScriptEvent('window', event);
    });
    function urlToFilePath(url) {
        if (!url)
            return null;
        try {
            new url_1.URL(url);
        }
        catch (err) {
            (0, ts_extras_1.assertError)(err);
            if (err.name !== 'TypeError')
                throw err;
            return null;
        }
        return relay.urlPathToFilePath(new url_1.URL(url).pathname);
    }
}
exports.attachBrowserScriptRelay = attachBrowserScriptRelay;
function replaceUrlsInStack(relay, stack) {
    if (!stack)
        return stack;
    return (stack
        // Safari
        .replace(/(?:^|\b)\S*@http:\/\/[^/]+\/__p5_server_static\/console-relay(?:\.min)\.js:\d+:\d+\n/g, '')
        // Chrome (first line, without parens)
        .replace(/(?:^|\b) *at[^\n]+?http:\/\/[^/]+\/__p5_server_static\/console-relay(?:\.min)\.js:\d+:\d+\n/g, '')
        // Chrome (subsequent lines, with parens)
        .replace(/ +at[^\n]+?\(http:\/\/[^/]+\/__p5_server_static\/console-relay(?:\.min)\.js:\d+:\d+\)\n/gm, '')
        // http:// -> file:///
        .replace(/\bhttps?:\/\/(?:localhost|127\.0\.0\.1)(?::\d+)?\/[^\s:]+/g, url => relay.serverUrlToFileUrl(url) || url));
}
exports.replaceUrlsInStack = replaceUrlsInStack;
function injectScriptEventRelayScript(html) {
    return (0, helpers_1.addScriptToHtmlHead)(html, `${constants_1.staticAssetPrefix}/console-relay.min.js`);
}
exports.injectScriptEventRelayScript = injectScriptEventRelayScript;
const serializationPrefix = '__p5_server_serialization_:';
// eslint-disable-next-line @typescript-eslint/no-explicit-any
function decodeUnserializableValue(value) {
    return typeof value === 'string' && value.startsWith(serializationPrefix)
        ? eval(value.slice(serializationPrefix.length))
        : value;
}
