"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createDirectoryListing = exports.defaultDirectoryExclusions = void 0;
const fs_1 = __importDefault(require("fs"));
const marked_1 = __importDefault(require("marked"));
const p5_analysis_1 = require("p5-analysis");
const path_1 = __importDefault(require("path"));
const pug_1 = __importDefault(require("pug"));
const helpers_1 = require("../helpers");
const constants_1 = require("./constants");
const templates_1 = require("./templates");
exports.defaultDirectoryExclusions = [
    '.*',
    '*~',
    '*.log',
    'node_modules',
    'package.json',
    'package-lock.json',
    // Linux
    '~*',
    // macOS
    'Icon\r',
    // Windows
    'Thumbs.db',
];
async function createDirectoryListing(dir, breadcrumbPath, options = {}) {
    const { staticMode, templateName, templateOptions } = {
        staticMode: false,
        templateName: 'directory.pug',
        templateOptions: {},
        ...options,
    };
    const { sketches, unassociatedFiles } = await p5_analysis_1.Sketch.analyzeDirectory(dir, {
        exclusions: exports.defaultDirectoryExclusions,
    });
    sketches.sort((a, b) => a.name.localeCompare(b.name));
    unassociatedFiles.sort((a, b) => a.toLowerCase().localeCompare(b.toLowerCase()));
    const readmeName = unassociatedFiles.find(name => /^readme\.(md|mkd|mkdn|mdwn|mdown|markdown)$/i.test(name));
    const readme = readmeName
        ? {
            name: readmeName,
            html: markdown(fs_1.default.readFileSync(path_1.default.join(dir, readmeName), 'utf-8')),
            url: staticMode ? `${readmeName}.html` : readmeName,
        }
        : null;
    const directories = unassociatedFiles.filter(s => fs_1.default.statSync(path_1.default.join(dir, s)).isDirectory());
    const files = unassociatedFiles.filter(s => !directories.includes(s) && s !== readmeName);
    const title = dir === './' ? 'P5 Server' : path_1.default.basename(dir);
    const templatePaths = ['', '.pug'].flatMap(ext => [
        path_1.default.join(templates_1.templateDir, templateName + ext),
        path_1.default.join(templates_1.templateDir, templateName, `directory${ext}`),
    ]);
    const templatePath = templatePaths.find(p => fs_1.default.existsSync(p) && !fs_1.default.statSync(p).isDirectory());
    if (!templatePath) {
        throw new Error(`Could not find template ${templateName} in ${templatePaths.join(', ')}`);
    }
    const pathComponents = (0, helpers_1.pathComponentsForBreadcrumbs)(breadcrumbPath || dir);
    return pug_1.default.renderFile(templatePath, {
        ...templateOptions,
        staticAssetPrefix: constants_1.staticAssetPrefix,
        directories,
        files,
        pathComponents,
        readme,
        sketches,
        title,
        // functions
        directory_index,
        path_to,
        path_to_src_view,
        markdown,
        play_link,
        // pug options
        cache: true,
        filters: { markdown },
    });
    function directory_index(dir) {
        return staticMode ? `${dir}/index.html` : `${dir}/`;
    }
    function markdown(md) {
        return md ? (0, marked_1.default)(md, templates_1.markedOptions) : '';
    }
    function path_to(filepath, sk) {
        return path_1.default.relative(dir, path_1.default.join(sk.dir, filepath));
    }
    function path_to_src_view(file, sk) {
        const filepath = path_to(file, sk);
        return staticMode
            ? `${filepath}.html`
            : filepath.match(/.*\.(html?|js)$/i)
                ? `${filepath}?fmt=view`
                : filepath;
    }
    function play_link(sk) {
        return path_to(staticMode && sk.structureType === 'script'
            ? sk.mainFile.replace(/\.js$/i, '.html')
            : sk.mainFile, sk);
    }
}
exports.createDirectoryListing = createDirectoryListing;
