"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.promiseClose = exports.promiseListen = void 0;
/** A wrapper for express.Application#listen that returns a Promise.
 *
 * The Promise succeeds if server.address() returns truthy within `interval` ms.
 *
 * It fails if the server sends on 'error' event or fails to produce and address
 * within the specified interval.
 * */
function promiseListen(app, port, timeout = 1000) {
    return new Promise((resolve, reject) => {
        const server = app.listen(port);
        server.on('error', onError);
        const timeoutTimer = setTimeout(() => {
            const address = server.address();
            removeListeners();
            if (address) {
                resolve(server);
            }
            else {
                reject(new Error('Failed to start server'));
            }
        }, timeout);
        const intervalTimer = setInterval(() => {
            const address = server.address();
            if (address) {
                removeListeners();
                resolve(server);
            }
        }, 50);
        function onError(e) {
            removeListeners();
            reject(e);
        }
        function removeListeners() {
            server.off('error', onError);
            clearTimeout(timeoutTimer);
            clearInterval(intervalTimer);
        }
    });
}
exports.promiseListen = promiseListen;
/** A wrapper for http.Server#close that returns a Promise.
 *
 * The Promise succeeds if the server sends a 'close' event within `interval`
 * ms.
 *
 * It fails if the server sends an 'error' event or fails to close within the
 * specified interval.
 */
function promiseClose(server, timeout = 10000) {
    return new Promise((resolve, reject) => {
        // server.close((err?: Error) => (err ? reject(err) : resolve()));
        server.close();
        server.on('close', onClose);
        server.on('error', onError);
        const timeoutTimer = setTimeout(() => {
            removeListeners();
            onError(new Error('Failed to close server within the timeout period'));
        }, timeout);
        function onClose() {
            removeListeners();
            resolve();
        }
        function onError(e) {
            removeListeners();
            reject(e);
        }
        function removeListeners() {
            server.off('close', onClose);
            server.off('error', onError);
            clearTimeout(timeoutTimer);
        }
    });
}
exports.promiseClose = promiseClose;
