"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createRouter = void 0;
const express_1 = __importDefault(require("express"));
const fs_1 = __importDefault(require("fs"));
const promises_1 = require("fs/promises");
const p5_analysis_1 = require("p5-analysis");
const path_1 = __importDefault(require("path"));
const helpers_1 = require("../helpers");
const ts_extras_1 = require("../ts-extras");
const browserScriptEventRelay_1 = require("./browserScriptEventRelay");
const constants_1 = require("./constants");
const directoryListing_1 = require("./directoryListing");
const liveReload_1 = require("./liveReload");
const proxyCache_1 = require("./proxyCache");
const templates_1 = require("./templates");
function createRouter(config) {
    const router = express_1.default.Router();
    router.get('/', async (req, res) => {
        const sketchFile = config.sketchFile;
        const file = path_1.default.join(config.root, decodeURIComponent(req.path));
        if (sketchFile) {
            if (await p5_analysis_1.Sketch.isSketchScriptFile(sketchFile)) {
                const sketch = await p5_analysis_1.Sketch.fromFile(sketchFile);
                sendHtml(req, res, await sketch.getHtmlContent());
            }
            else {
                sendHtml(req, res, await (0, promises_1.readFile)(sketchFile, 'utf-8'));
            }
        }
        else if (config.screenshot) {
            const { sketches } = fs_1.default.statSync(file).isDirectory()
                ? await p5_analysis_1.Sketch.analyzeDirectory(file, {
                    exclusions: directoryListing_1.defaultDirectoryExclusions
                })
                : { sketches: [] };
            if (sketches.length !== 1)
                throw new Error(`Expected exactly one sketch in ${file}`);
            const [sketch] = sketches;
            const html = sketch.htmlFile
                ? await (0, promises_1.readFile)(sketch.htmlFilePath, 'utf-8')
                : await sketch.getHtmlContent();
            sendHtml(req, res, html);
        }
        else {
            await sendDirectoryListing(config, req, res);
        }
    });
    router.post('/__p5_server/screenshot', express_1.default.json({ limit: '50mb' }), async (req, res) => {
        const { dataURL } = req.body;
        const m = dataURL.match(/^data:image\/(.+?);base64,(.*)$/);
        if (!m || !config.screenshot?.onFrameData)
            return res.sendStatus(200);
        await config.screenshot.onFrameData({
            imageType: m[1],
            data: Buffer.from(m[2], 'base64'),
            frameNumber: req.body.frameNumber
        });
        res.sendStatus(200);
    });
    router.get('/*.html?', (req, res, next) => {
        const file = path_1.default.join(config.root, decodeURIComponent(req.path));
        try {
            if (req.query.fmt === 'view') {
                res.set('Content-Type', 'text/plain');
                res.sendFile(req.path, { root: config.root });
                return;
            }
            if (req.headers['accept']?.match(/\btext\/html\b/)) {
                sendHtml(req, res, fs_1.default.readFileSync(file, 'utf-8'));
                return;
            }
        }
        catch (err) {
            (0, ts_extras_1.assertError)(err);
            if (err.code !== 'ENOENT') {
                throw err;
            }
        }
        next();
    });
    // A request for the HTML of a JavaScript file returns HTML that includes the sketch.
    // A request for the HTML of a main sketch js file redirects to the sketch's index page.
    router.get('/*.js', async (req, res, next) => {
        const filepath = path_1.default.join(config.root, decodeURIComponent(req.path));
        // bare-javascript sketch; not view source
        if (req.headers['accept']?.match(/\btext\/html\b/) &&
            req.query.fmt !== 'view' &&
            (await p5_analysis_1.Sketch.isSketchScriptFile(filepath))) {
            const { sketches } = await p5_analysis_1.Sketch.analyzeDirectory(path_1.default.dirname(filepath), {
                exclusions: directoryListing_1.defaultDirectoryExclusions
            });
            const sketch = sketches.find(sketch => sketch.files.includes(path_1.default.basename(filepath)));
            if (sketch) {
                return sendHtml(req, res, await sketch.getHtmlContent());
            }
        }
        // view source
        if (req.headers['accept']?.match(/\btext\/html\b/) && req.query.fmt === 'view') {
            const source = await (0, promises_1.readFile)(filepath, 'utf-8');
            const title = req.path.replace(/^\//, '');
            const html = (0, templates_1.sourceViewTemplate)({ source, title });
            res.set('Content-Type', 'text/html');
            return res.send(html);
        }
        try {
            const errs = p5_analysis_1.Script.fromFile(filepath).getErrors();
            if (errs.length) {
                console.error(`Syntax error in ${filepath}: ${errs[0].message}`);
                res.set('Content-Type', 'text/html');
                return res.send((0, templates_1.createSyntaxErrorJsReporter)(filepath, errs));
            }
        }
        catch (err) {
            (0, ts_extras_1.assertError)(err);
            if (err.code === 'ENOENT') {
                return next();
            }
            else {
                throw err;
            }
        }
        next();
    });
    router.get('/*.md', (req, res, next) => {
        if (req.headers['accept']?.match(/\btext\/html\b/)) {
            const file = path_1.default.join(config.root, decodeURIComponent(req.path));
            if (!fs_1.default.existsSync(file)) {
                return next();
            }
            const data = fs_1.default.readFileSync(file, 'utf-8');
            res.set('Content-Type', 'text/html');
            let html = (0, templates_1.markdownToHtmlPage)(data);
            if (config.proxyCache)
                html = (0, proxyCache_1.replaceUrlsInHtml)(html);
            return res.send(html);
        }
        return next();
    });
    router.get('*', (req, res, next) => {
        if (req.headers['accept']?.match(/\btext\/html\b/)) {
            const file = path_1.default.join(config.root, decodeURIComponent(req.path));
            if (fs_1.default.existsSync(file) && fs_1.default.statSync(file).isDirectory()) {
                return sendDirectoryListing(config, req, res);
            }
        }
        next();
    });
    return router;
    function sendHtml(req, res, html) {
        html = (0, liveReload_1.injectLiveReloadScript)(html, req.app.locals.liveReloadServer);
        if (config.relayConsoleMessages ||
            'send-console-messages' in req.query ||
            'vscodeBrowserReqId' in req.query) {
            html = (0, browserScriptEventRelay_1.injectScriptEventRelayScript)(html);
        }
        if (config.screenshot && req.path === '/') {
            html = (0, helpers_1.addScriptToHtmlHead)(html, `${constants_1.staticAssetPrefix}/screenshot.min.js`);
            html = (0, helpers_1.addScriptToHtmlHead)(html, {
                __p5_server_screenshot_settings: config.screenshot
            });
        }
        if (config.proxyCache)
            html = (0, proxyCache_1.replaceUrlsInHtml)(html);
        res.set('Content-Type', 'text/html');
        res.send(html);
    }
}
exports.createRouter = createRouter;
async function sendDirectoryListing(config, req, res) {
    // This is needed for linked files to work.
    if (!req.originalUrl.endsWith('/')) {
        return res.redirect(req.originalUrl + '/');
    }
    const dir = path_1.default.join(config.root, decodeURIComponent(req.path).replace(/\//g, path_1.default.sep));
    // read the directory contents
    const indexFile = (await (0, promises_1.readdir)(dir)).find(file => /^index\.html?$/i.test(file));
    let html = indexFile
        ? await (0, promises_1.readFile)(path_1.default.join(dir, indexFile), 'utf-8')
        : await (0, directoryListing_1.createDirectoryListing)(dir, req.originalUrl, {
            templateName: config.theme
        });
    // Note: This injects the reload script into both static and generated index
    // pages. This ensures that the index page reloads when the directory contents
    // change.
    if (config.liveServer) {
        html = (0, liveReload_1.injectLiveReloadScript)(html, req.app.locals.liveReloadServer);
    }
    if (config.proxyCache)
        html = (0, proxyCache_1.replaceUrlsInHtml)(html);
    return res.send(html);
}
